package com.perforce.hws.plugin.perforce.routes;

import com.perforce.hws.core.p4base.HasGson;
import com.perforce.hws.core.p4base.P4Methods;
import com.perforce.hws.core.p4base.ResultMap;
import com.perforce.hws.core.p4base.ResultMapMethods;
import com.perforce.hws.core.sessions.SessionData;
import com.perforce.hws.route.UsesJsonRequests;
import com.perforce.hws.utils.HWSSettings;
import com.perforce.hws.utils.ModelConversion;
import com.perforce.hws.utils.UsesServerHandles;
import com.perforce.p4client.models.BranchCommand;

/**
 * PATCH /{server}/branches/{branch}
 */
public class PerforceUpdateBranchRoute implements
        spark.Route,
        P4Methods,
        ResultMapMethods,
        HasGson,
        ModelConversion,
        UsesJsonRequests,
        UsesServerHandles {

    @Override
    public Object handle(spark.Request request, spark.Response response) throws Exception {
        SessionData sessionData = request.attribute("sessionData");
        HWSSettings settings = request.attribute("settings");

        String server = request.params("server");
        String branch = request.params("branch");

        BranchCommand branchCommand = readModel(request, BranchCommand.class);

        java.util.Map<String, Object> updates = toRequestMap(branchCommand);

        return withServerHandle(server, sessionData, settings, serverHandle -> {
            java.util.List<ResultMap> results = exec(serverHandle, "branch", "-o", branch);
            java.util.Map<String, Object> current = collateArrayEntries(results.get(0));

            current.putAll(updates);

            java.util.Map<String, Object> toUpdate = divideArrayEntries(current);

            results = execInput(serverHandle, toUpdate, "branch", "-i");

            return toCommandResponse(results);
        });
    }
}
